@@ -2,6 +2,7 @@ import lightning as L
 import torch
 from torch.utils.data import DataLoader, random_split, TensorDataset
 from typing import Optional
+from spotpython.utils.split import calculate_data_split
 
 
 class LightDataModule(L.LightningDataModule):
@@ -95,6 +96,29 @@ class LightDataModule(L.LightningDataModule):
         self.scaler = scaler
         self.verbosity = verbosity
 
+    def transform_dataset(self, dataset) -> TensorDataset:
+        """Applies the scaler transformation to the dataset.
+
+        Args:
+            dataset (List[Tuple[torch.Tensor, Any]]): The dataset to transform, consisting of data and target pairs.
+
+        Returns:
+            TensorDataset: A PyTorch TensorDataset containing the transformed and cloned data and targets.
+
+        Raises:
+            ValueError: If the input data is not correctly formatted for transformation.
+        """
+        try:
+            # Perform transformations on the data in a single iteration
+            transformed_data = [(self.scaler.transform(data), target) for data, target in dataset]
+            # Clone and detach data tensors
+            data_tensors = [data.clone().detach() for data, _ in transformed_data]
+            target_tensors = [target.clone().detach() for _, target in transformed_data]
+            # Create a TensorDataset from the processed data
+            return TensorDataset(torch.stack(data_tensors).squeeze(1), torch.stack(target_tensors))
+        except Exception as e:
+            raise ValueError(f"Error transforming dataset: {e}")
+
     def prepare_data(self) -> None:
         """Prepares the data for use."""
         # download
@@ -124,25 +148,12 @@ class LightDataModule(L.LightningDataModule):
                 Training set size: 3
 
         """
-        # if test_size is float, then train_size is 1 - test_size
-        test_size = self.test_size
-        if isinstance(self.test_size, float):
-            full_train_size = round(1.0 - test_size, 2)
-            val_size = round(full_train_size * test_size, 2)
-            train_size = round(full_train_size - val_size, 2)
-        else:
-            # if test_size is int, then train_size is len(data_full) - test_size
-            full_train_size = len(self.data_full) - test_size
-            val_size = int(full_train_size * test_size / len(self.data_full))
-            train_size = full_train_size - val_size
-
-        if self.verbosity > 0:
-            print(f"LightDataModule.setup(): stage: {stage}")
-        if self.verbosity > 1:
-            print(f"LightDataModule setup(): full_train_size: {full_train_size}")
-            print(f"LightDataModule setup(): val_size: {val_size}")
-            print(f"LightDataModule setup(): train_size: {train_size}")
-            print(f"LightDataModule setup(): test_size: {test_size}")
+        full_train_size, val_size, train_size, test_size = calculate_data_split(
+            test_size=self.test_size,
+            full_size=len(self.data_full),
+            verbosity=self.verbosity,
+            stage=stage,
+        )
 
         # Assign train/val datasets for use in dataloaders
         if stage == "fit" or stage is None:
@@ -153,64 +164,37 @@ class LightDataModule(L.LightningDataModule):
                 self.data_full, [train_size, val_size, test_size], generator=generator_fit
             )
             if self.scaler is not None:
-                # Fit the scaler on training data and transform both train and val data
+                # Fit the scaler on training data
                 scaler_train_data = torch.stack([self.data_train[i][0] for i in range(len(self.data_train))]).squeeze(1)
-                # train_val_data = self.data_train[:,0]
                 if self.verbosity > 0:
                     print(scaler_train_data.shape)
                 self.scaler.fit(scaler_train_data)
-                self.data_train = [(self.scaler.transform(data), target) for data, target in self.data_train]
-                data_tensors_train = [data.clone().detach() for data, target in self.data_train]
-                target_tensors_train = [target.clone().detach() for data, target in self.data_train]
-                self.data_train = TensorDataset(
-                    torch.stack(data_tensors_train).squeeze(1), torch.stack(target_tensors_train)
-                )
-                # print(self.data_train)
-                self.data_val = [(self.scaler.transform(data), target) for data, target in self.data_val]
-                data_tensors_val = [data.clone().detach() for data, target in self.data_val]
-                target_tensors_val = [target.clone().detach() for data, target in self.data_val]
-                self.data_val = TensorDataset(torch.stack(data_tensors_val).squeeze(1), torch.stack(target_tensors_val))
+                # Transform the training data
+                self.data_train = self.transform_dataset(self.data_train)
+                # Transform the validation data
+                self.data_val = self.transform_dataset(self.data_val)
 
         # Assign test dataset for use in dataloader(s)
         if stage == "test" or stage is None:
             if self.verbosity > 0:
                 print(f"test_size: {test_size} used for test dataset.")
-            # get test data set as test_abs percent of the full dataset
             generator_test = torch.Generator().manual_seed(self.test_seed)
             self.data_test, _ = random_split(self.data_full, [test_size, full_train_size], generator=generator_test)
             if self.scaler is not None:
-                self.data_test = [(self.scaler.transform(data), target) for data, target in self.data_test]
-                data_tensors_test = [data.clone().detach() for data, target in self.data_test]
-                target_tensors_test = [target.clone().detach() for data, target in self.data_test]
-                self.data_test = TensorDataset(
-                    torch.stack(data_tensors_test).squeeze(1), torch.stack(target_tensors_test)
-                )
-
-        # if stage == "predict" or stage is None:
-        #     print(f"test_size, full_train_size: {test_size}, {full_train_size}")
-        #     generator_predict = torch.Generator().manual_seed(self.test_seed)
-        #     full_data_predict, _ = random_split(
-        #         self.data_full, [test_size, full_train_size], generator=generator_predict
-        #     )
-        #     # Only keep the features for prediction
-        #     self.data_predict = [x for x, _ in full_data_predict]
+                # Transform the test data
+                self.data_test = self.transform_dataset(self.data_test)
 
         # Assign pred dataset for use in dataloader(s)
         if stage == "predict" or stage is None:
             if self.verbosity > 0:
                 print(f"test_size: {test_size} used for predict dataset.")
-            # get test data set as test_abs percent of the full dataset
             generator_predict = torch.Generator().manual_seed(self.test_seed)
             self.data_predict, _ = random_split(
                 self.data_full, [test_size, full_train_size], generator=generator_predict
             )
             if self.scaler is not None:
-                self.data_predict = [(self.scaler.transform(data), target) for data, target in self.data_predict]
-                data_tensors_predict = [data.clone().detach() for data, target in self.data_predict]
-                target_tensors_predict = [target.clone().detach() for data, target in self.data_predict]
-                self.data_predict = TensorDataset(
-                    torch.stack(data_tensors_predict).squeeze(1), torch.stack(target_tensors_predict)
-                )
+                # Transform the predict data
+                self.data_predict = self.transform_dataset(self.data_predict)
 
     def train_dataloader(self) -> DataLoader:
         """
@@ -235,7 +219,6 @@ class LightDataModule(L.LightningDataModule):
             print(f"LightDataModule.train_dataloader(). data_train size: {len(self.data_train)}")
         # print(f"LightDataModule: train_dataloader(). batch_size: {self.batch_size}")
         # print(f"LightDataModule: train_dataloader(). num_workers: {self.num_workers}")
-        # apply fit_transform to the training data
         return DataLoader(self.data_train, batch_size=self.batch_size, num_workers=self.num_workers)
 
     def val_dataloader(self) -> DataLoader:
@@ -260,7 +243,6 @@ class LightDataModule(L.LightningDataModule):
             print(f"LightDataModule.val_dataloader(). Val. set size: {len(self.data_val)}")
         # print(f"LightDataModule: val_dataloader(). batch_size: {self.batch_size}")
         # print(f"LightDataModule: val_dataloader(). num_workers: {self.num_workers}")
-        # apply fit_transform to the val data
         return DataLoader(self.data_val, batch_size=self.batch_size, num_workers=self.num_workers)
 
     def test_dataloader(self) -> DataLoader:
@@ -312,6 +294,4 @@ class LightDataModule(L.LightningDataModule):
             print(f"LightDataModule.predict_dataloader(). Predict set size: {len(self.data_predict)}")
         # print(f"LightDataModule: predict_dataloader(). batch_size: {self.batch_size}")
         # print(f"LightDataModule: predict_dataloader(). num_workers: {self.num_workers}")
-        # apply fit_transform to the val data
-
         return DataLoader(self.data_predict, batch_size=len(self.data_predict), num_workers=self.num_workers)
